<?php
/**
 * @package		Custom Canonical CK
 * @version		1.0.0
 * @copyright 2022 Cédric KEIFLIN
 * @author Cédric KEIFLIN <contact@joomlack.fr>
 * 
 * Original plugin from
 * @copyright	Copyright (C) 2018 DanielMorell.com. All rights reserved. 
 * @license		http://www.gnu.org/licenses/gpl-3.0.html GNU/GPLv3 or later
 */

// Check to ensure this file is included in Joomla!
defined( '_JEXEC' ) or die( 'Restricted access' );									   			  

use Joomla\CMS\Router\Route;
use Joomla\Component\Content\Site\Helper\RouteHelper;
use Joomla\CMS\Uri\Uri;

class plgSystemCustomcanonicalck extends \Joomla\CMS\Plugin\CMSPlugin {

	protected $autoloadLanguage = true;

	protected $canonical = null;

	protected $loaded = false;

	private $shallLoad = true;

	function __construct (&$subject, $config) {
		parent::__construct($subject, $config);
	}

	/* 
	 * Initiate the plugin load
	 *
	 * Return mixed
	 */
	function registerListeners() {
		if ($this->shallLoad() === true) {
			parent::registerListeners();
		} else {
			return false;
		}
	}

	private function shallLoad() {
		if (! $this->shallLoad) {
			return false;
		}
		$app = \Joomla\CMS\Factory::getApplication();
		if (! $app->isClient('site') && ! $app->isClient('administrator'))
		{
			return false;
		}

		$input = $app->input;
		// exit if we are in one of these cases
		if ($input->get('option', '', 'string') == 'com_ajax' 
			|| $input->get('option', '', 'string') == 'com_media'
			|| $input->get('format', '', 'string') == 'raw'
			) {
			return false;
		}

		return true;
	}

	//Add canonical fields to Joomla back-end
	public function onContentPrepareForm($form, $data) {
		// Include the helper functions only once
		require_once dirname(__FILE__) . '/helper.php';

		$helper = new PlgCustomcanonicalckHelper();
		$plg_path  = \Joomla\CMS\Uri\Uri::root(true).'/plugins/system/customcanonicalck';
		$article   = $this->params->get('article', '');
		$category  = $this->params->get('category', '');
		$tag       = $this->params->get('tag', '');
		$menu_item = $this->params->get('menu_item', '');

		\Joomla\CMS\Form\Form::addFormPath(\JPATH_PLUGINS.'/system/customcanonicalck/params');

		if (is_array($data)) {
			$data = $this->arrayToObject($data);
		}

		// load jQuery
		\Joomla\CMS\HTML\HTMLHelper::_('jquery.framework', true);

		//Article canonical
		if ($article == '1' && $form->getName()=='com_content.article') {
			$form->loadFile('article', false);
			$helper->autoLoadCanonicalBtn('article', $data);
		}
		
		//Category canonical
		if ($category == '1' && $form->getName()=='com_categories.categorycom_content') {
			$form->loadFile('category', false);
			$helper->autoLoadCanonicalBtn('category', $data);
		}

		//Tag canonical
		if ($tag == '1' && $form->getName()=='com_tags.tag') {
			$form->loadFile('tag', false);
			$test = $helper->autoLoadCanonicalBtn('tag', $data);
		}
		
		//Menu item canonical
		if ($menu_item == '1' && $form->getName() == 'com_menus.item') {
			$form->loadFile('menu_item', false);
			$test = $helper->autoLoadCanonicalBtn('menu', $data);
		}
	}
	
	//Remove Joomla's canonical
	function onBeforeCompileHead() {
		$app = \Joomla\CMS\Factory::getApplication();
		if ($app->isClient('administrator')) {
			return;
		}
		
		if ($this->params->get('remove_core_canonical') == '1'){
			$doc = \Joomla\CMS\Factory::getDocument();
			foreach ( $doc->_links as $k => $array ) {
				if ( $array['relation'] == 'canonical' ) {
					unset($doc->_links[$k]);
				}
			}
		}
	}
	
	/**
	*	$context : com_phocacart.item
	*	$item : the product
	*	$params
	*	
	*	return @void
	*/
	public function onPCVonItemBeforeHeader($context, &$item, &$params) {
		/*
		if (! class_exists('PhocaCartLoader')) {
			require_once( JPATH_ADMINISTRATOR.'/components/com_phocacart/libraries/loader.php');
		}
		*/
		// only process in the product page
		if ($context !== 'com_phocacart.item') return;
		// check that the product exist
		if (! isset($item[0])) return;

		$id = $item[0]->id;
		$catid = $item[0]->preferred_catid;
		// do not change the canonical url if there is no preferred category set
		if (!$catid) return;

		$idAlias = $item[0]->alias;
		$url = Route::_(PhocacartRoute::getItemRoute((int)$id, (int)$catid, $idAlias));

		if ($url) $this->addCustomCanonical($url);
	}

	private function addCustomCanonical($url) {
		if (!$url) return;

		$canonicalLink = '<link href="' . $url . '" rel="canonical" />';
		$doc     = \Joomla\CMS\Factory::getDocument();
		$doc->addCustomTag($canonicalLink);
		$this->loaded = true;
	}

	//Place custom canonical on Joomla front-end
	public function onContentPrepare($context, &$item, &$params, $page) {
		// Include the helper functions only once
		require_once dirname(__FILE__) . '/helper.php';

		$helper = new PlgCustomcanonicalckHelper();

		static $is_set = false;
		if ($is_set) {
			return;
		}
		$is_set = true;
		
		$doc     = \Joomla\CMS\Factory::getDocument();
		$app     = \Joomla\CMS\Factory::getApplication();
		if ($app->isClient('administrator')) {
			return;
		}
		$option = $app->input->get('option');
		$view   = $app->input->get('view');

		//Get article canonical
		if ($context == "com_content.article" && ($option == "com_content" && $view == "article") && $this->params->get('article') == '1') {

			$attribs = json_decode($item->attribs);

			if(isset($attribs->canonical_url) && ($attribs->canonical_url !== '')) {

				$canonical = trim($attribs->canonical_url);

				if ($page > 0){
					$canonical = ''. $canonical .'?showall=&start='.$page.'';
				}
			}
		}

		//Get category canonical
		if ($context == "com_content.category" && ($option == "com_content" && $view == "category") && $this->params->get('category') == '1') {

			$page = $app->input->get('limitstart', 0, 'uint');

			$catid =  $app->input->get('id');

			// $basePath = JPATH_ADMINISTRATOR . '/components/com_categories';
			// require_once $basePath . '/models/category.php';
			// $category = new CategoriesModelCategory(array('table_path' => $basePath . '/tables'));

			$factory = $app->bootComponent('com_categories')->getMVCFactory();
			// Get an instance of the generic articles model
			$category = $factory->createModel('Category', 'Administrator', ['ignore_request' => true]);

			$category_params = $category->getItem($catid)->params;

			if (isset($category_params['canonical_url']) && ($category_params['canonical_url'] !== '')) {

				$canonical = trim($category_params['canonical_url']);

				if ($page > 0){
					$canonical = ''. $canonical .'?start='.$page.'';
				}
			}
		}

		//Get tag canonical
		// use com_tag also because it seems that there is a context issue somewhere in the core
		if (($context == "com_tag.tag" || $context == "com_tags.tag") && ($option == "com_tags" && $view == "tag") && $this->params->get('tag') =='1') {

			$tagid = $app->input->get('id', 0, 'int');
			if (is_array($tagid)) $tagid = implode(',', $tagid);
			// $basePath = JPATH_BASE . '/components/com_tags';
			// require_once $basePath . '/models/tag.php';
			// $tag = new TagsModelTag(array('table_path' => $basePath . '/tables'));

			$factory = $app->bootComponent('com_tags')->getMVCFactory();
			// Get an instance of the generic articles model
			$tag = $factory->createModel('Tag', 'Site', ['ignore_request' => true]);
			$tag_details = $tag->getItem($tagid)['0'];
			$tag_params = json_decode($tag_details->params);

			if (isset($tag_params->canonical_url) && ($tag_params->canonical_url !== '')) {

				$canonical = trim($tag_params->canonical_url);
			}
		}

		//Get menu item canonical
		// nothing automatic here because the canonical from the menu will override the automatic route url from the view
		if (($this->params->get('menu_item') =='1')) {
			$currentMenuItem = $app->getMenu()->getActive();
			$currentMenuItemParams = $currentMenuItem->getParams();
			$item_params = json_decode($currentMenuItemParams);
			
			if (isset($item_params->canonical_url) && ($item_params->canonical_url !== '') && !isset($canonical)) {

				$canonical = trim($item_params->canonical_url);
			}
		}

		// Add canonical link to document
		if (isset($canonical) && $canonical != '') {

			$canonical = $helper->buildUrl($canonical,$this->params);
			$canonicalLink = '<link href="' . $canonical . '" rel="canonical" />';

			$doc->addCustomTag($canonicalLink);
			$this->loaded = true;
		}
	}

	public function onBeforeRender() {
		if ($this->loaded) return;

		if (! $this->shallLoad()) return;

		$document = \Joomla\CMS\Factory::getApplication()->getDocument();
		$doctype = $document->getType();

		if ($doctype !== 'html')
		{
			return false;
		}

		// $doc     = \Joomla\CMS\Factory::getDocument();
		$app     = \Joomla\CMS\Factory::getApplication();
		if ($app->isClient('administrator')) {
			return;
		}
	
		//Get menu item canonical
		if (($this->params->get('menu_item') =='1')) {
			$currentMenuItem = $app->getMenu()->getActive();
			$currentMenuItemParams = $currentMenuItem->getParams();
			$item_params = json_decode($currentMenuItemParams);
			
			if (isset($item_params->canonical_url) && ($item_params->canonical_url !== '') && !isset($canonical)) {
				$canonical = trim($item_params->canonical_url);
			}
		}

		// Add canonical link to document
		if (isset($canonical) && $canonical != '') {

			// Include the helper functions only once
			require_once dirname(__FILE__) . '/helper.php';

			$helper = new PlgCustomcanonicalckHelper();
			$doc     = \Joomla\CMS\Factory::getDocument();
			$canonical = $helper->buildUrl($canonical,$this->params);
			$canonicalLink = '<link href="' . $canonical . '" rel="canonical" />';
			$doc->addCustomTag($canonicalLink);
			$this->loaded = true;
		}
	}

	private function arrayToObject($arr) {
		$object = new stdClass();
		foreach ($arr as $key => $value)
		{
			$object->$key = $value;
		}
		return $object;
	}
}